import { Widget } from "../model/Widget";
import { ElectronUtils } from "../utils/ElectronUtils";
import { WidgetPackage } from "../model/WidgetPackage";
import { Channel } from "./Channel";
import { UrlUtils } from "../utils/UrlUtils";
import { BroadcastEvent } from "../model/event/BroadcastEvent";
import { BroadcastApi } from "./BroadcastApi";
import localforage from "localforage";
export class WidgetApi {
    static async registerWidgets(widgets) {
        var _a;
        await ((_a = ElectronUtils.getAPI()) === null || _a === void 0 ? void 0 : _a.invoke(Channel.WIDGET, this.REGISTER_WIDGETS, JSON.stringify(widgets)));
    }
    static async registerWidgetPackage(widgetPackage) {
        var _a;
        await ((_a = ElectronUtils.getAPI()) === null || _a === void 0 ? void 0 : _a.invoke(Channel.WIDGET, this.REGISTER_WIDGET_PACKAGE, JSON.stringify(widgetPackage)));
    }
    static async getWidgets() {
        var _a;
        const data = await ((_a = ElectronUtils.getAPI()) === null || _a === void 0 ? void 0 : _a.invoke(Channel.WIDGET, this.GET_WIDGETS));
        const widgets = [];
        if (data) {
            const arr = JSON.parse(data);
            for (const i in arr) {
                widgets.push(Widget.parseObject(arr[i]));
            }
        }
        return widgets;
    }
    static async getWidgetPackages() {
        var _a;
        return await ((_a = ElectronUtils.getAPI()) === null || _a === void 0 ? void 0 : _a.invoke(Channel.WIDGET, this.GET_WIDGET_PACKAGES));
    }
    /**
     *
     * @param name package name
     */
    static async getWidget(name) {
        var _a;
        return Widget.parseObject(await ((_a = ElectronUtils.getAPI()) === null || _a === void 0 ? void 0 : _a.invoke(Channel.WIDGET, this.GET_WIDGET, name)));
    }
    /**
     *
     * @param name package name
     */
    static async getWidgetPackage(name) {
        var _a;
        return WidgetPackage.parseObject(await ((_a = ElectronUtils.getAPI()) === null || _a === void 0 ? void 0 : _a.invoke(Channel.WIDGET, this.GET_WIDGET_PACKAGE, name)));
    }
    /**
     * 移除组件
     * @param id
     */
    static async removeHostedWidget(id) {
        var _a;
        return (_a = ElectronUtils.getAPI()) === null || _a === void 0 ? void 0 : _a.invoke(Channel.WIDGET, this.REMOVE_HOSTED_WIDGET, id);
    }
    /**
     * 通过组件名移除已添加的组件
     * @param name  组件名
     */
    static async removeHostedWidgetByName(name) {
        var _a;
        return (_a = ElectronUtils.getAPI()) === null || _a === void 0 ? void 0 : _a.invoke(Channel.WIDGET, this.REMOVE_HOSTED_WIDGET_BY_NAME, name);
    }
    /**
     * 获取已添加的组件
     * @param name  组件名，可以不传
     */
    static async getHostedWidgets(name) {
        var _a;
        return (_a = ElectronUtils.getAPI()) === null || _a === void 0 ? void 0 : _a.invoke(Channel.WIDGET, this.GET_HOSTED_WIDGETS, name);
    }
    static async openDevTools(widgetId) {
        var _a;
        return (_a = ElectronUtils.getAPI()) === null || _a === void 0 ? void 0 : _a.invoke(Channel.WIDGET, this.OPEN_DEV_TOOLS, widgetId);
    }
    /**
     * 获取组件配置地址
     * @param widgetName
     */
    static async getWidgetConfigUrl(widgetName, widgetParams) {
        const widget = await this.getWidget(widgetName);
        if (!widget || widget.configUrl == null)
            return null;
        const widgetPackage = await this.getWidgetPackage(widget.packageName);
        if (!widgetPackage)
            return null;
        return UrlUtils.getWidgetUrl(widget.configUrl, widgetPackage, widgetParams);
    }
    static async getWidgetUrl(widgetName, widgetParams) {
        const widget = await this.getWidget(widgetName);
        if (!widget || widget.configUrl == null)
            return null;
        const widgetPackage = await this.getWidgetPackage(widget.packageName);
        if (!widgetPackage)
            return null;
        return UrlUtils.getWidgetUrl(widget.url, widgetPackage, widgetParams);
    }
    static async getWidgetPackageIndexUrl(packageName, hash) {
        const widgetPackage = await this.getWidgetPackage(packageName);
        if (!widgetPackage)
            return null;
        return widgetPackage.getIndexUrl(hash);
    }
    static async getWidgetPackageUrl(packageName) {
        const widgetPackage = await this.getWidgetPackage(packageName);
        if (!widgetPackage)
            return null;
        return widgetPackage.url;
    }
    /**
     * 通过组件名保存组件信息，通常用于存储可以在同类组件中共用的数据
     * @param data
     * @param options
     */
    static async saveDataByName(data, options = { sendBroadcast: true }) {
        const store = this.getStore(data.name);
        const json = JSON.stringify(data);
        const result = await store.setItem(data.name, json);
        if (options.sendBroadcast) {
            const broadcastEvent = new BroadcastEvent(BroadcastEvent.TYPE_WIDGET_UPDATED, "", {
                name: data.name,
                id: options.id,
                json
            });
            await BroadcastApi.sendBroadcastEvent(broadcastEvent);
        }
        return result;
    }
    /**
     * 获取组件 LocalForage 存储实例
     * @param name
     */
    static getStore(name) {
        if (this.stores.has(name)) {
            return this.stores.get(name);
        }
        const store = localforage.createInstance({ name: name });
        this.stores.set(name, store);
        return store;
    }
}
WidgetApi.REGISTER_WIDGETS = "register-widgets";
WidgetApi.REGISTER_WIDGET_PACKAGE = "register-widget-package";
WidgetApi.GET_WIDGETS = "get-widgets";
WidgetApi.GET_WIDGET = "get-widget";
WidgetApi.GET_WIDGET_PACKAGE = "get-widget-package";
WidgetApi.GET_WIDGET_PACKAGES = "get-widget-packages";
WidgetApi.REMOVE_HOSTED_WIDGET = "remove-hosted-widget";
WidgetApi.REMOVE_HOSTED_WIDGET_BY_NAME = "remove-hosted-widget-by-name";
WidgetApi.GET_HOSTED_WIDGETS = "get-hosted-widgets";
WidgetApi.OPEN_DEV_TOOLS = "open-dev-tools";
WidgetApi.stores = new Map();
